@@ -155,7 +155,7 @@ class Campaign:
         else:
             return results
 
-    def __getitem__(self, index):
+    def __getitem__(self, index) -> RDYFile:
         return self.files[index]
 
     def __len__(self):
@@ -417,7 +417,7 @@ class Campaign:
         """
         if self.osm:
             for f in tqdm(self):
-                if f.matched_ways and f.matched_ways:
+                if f.matched_ways and f.matched_nodes:
                     if rematch:
                         logger.info("(%s) File already has a map matching! Rematching..." % f.filename)
                         f.do_map_matching(**kwargs)
